package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.StreamCipher;
import cn.com.infosec.crypto.params.KeyParameter;
import cn.com.infosec.crypto.params.ParametersWithIV;

public class HC256Engine implements StreamCipher {
   private int[] p = new int[1024];
   private int[] q = new int[1024];
   private int cnt = 0;
   private byte[] key;
   private byte[] iv;
   private boolean initialised;
   private byte[] buf = new byte[4];
   private int idx = 0;

   private int step() {
      int var10000 = this.cnt;
      int var1 = var10000 & 1023;
      int[] var10001;
      int[] var2;
      int var3;
      int var4;
      int var5;
      int var6;
      int[] var7;
      int[] var8;
      if (var10000 < 1024) {
         var10001 = var2 = this.p;
         var3 = var2[var1 - 3 & 1023];
         var4 = var2[var1 - 1023 & 1023];
         var5 = var2[var1];
         var6 = var10001[var1 - 10 & 1023] + (rotateRight(var3, 10) ^ rotateRight(var4, 23));
         var8 = var7 = this.q;
         var2[var1] = var5 + var6 + var7[(var3 ^ var4) & 1023];
         var10000 = var8[(var3 = (var2 = this.p)[var1 - 12 & 1023]) & 255] + var7[(var3 >> 8 & 255) + 256] + var7[(var3 >> 16 & 255) + 512] + var7[(var3 >> 24 & 255) + 768] ^ var2[var1];
      } else {
         var10001 = var2 = this.q;
         var3 = var2[var1 - 3 & 1023];
         var4 = var2[var1 - 1023 & 1023];
         var5 = var2[var1];
         var6 = var10001[var1 - 10 & 1023] + (rotateRight(var3, 10) ^ rotateRight(var4, 23));
         var8 = var7 = this.p;
         var2[var1] = var5 + var6 + var7[(var3 ^ var4) & 1023];
         var10000 = var8[(var3 = (var2 = this.q)[var1 - 12 & 1023]) & 255] + var7[(var3 >> 8 & 255) + 256] + var7[(var3 >> 16 & 255) + 512] + var7[(var3 >> 24 & 255) + 768] ^ var2[var1];
      }

      this.cnt = this.cnt + 1 & 2047;
      return var10000;
   }

   private void init() {
      // $FF: Couldn't be decompiled
   }

   private byte getByte() {
      int var1;
      if (this.idx == 0) {
         var1 = this.step();
         byte[] var2;
         byte[] var10000 = var2 = this.buf;
         var2[3] = (byte)(var1 & 255);
         int var10001 = var1 >> 8;
         var2[2] = (byte)(var1 >> 8 & 255);
         var10001 >>= 8;
         var2[1] = (byte)(var10001 & 255);
         var10000[0] = (byte)(var10001 >> 8 & 255);
      }

      byte var3 = this.buf[var1 = this.idx];
      this.idx = var1 + 1 & 3;
      return var3;
   }

   private static int rotateRight(int x, int bits) {
      return x >>> bits | x << -bits;
   }

   public String getAlgorithmName() {
      return "HC-256";
   }

   public void init(boolean forEncryption, CipherParameters params) {
      CipherParameters forEncryption1;
      if (params instanceof ParametersWithIV) {
         ParametersWithIV var10000 = (ParametersWithIV)params;
         this.iv = ((ParametersWithIV)params).getIV();
         forEncryption1 = var10000.getParameters();
      } else {
         this.iv = new byte[0];
         forEncryption1 = params;
      }

      if (forEncryption1 instanceof KeyParameter) {
         this.key = ((KeyParameter)forEncryption1).getKey();
         this.init();
         this.initialised = true;
      } else {
         throw new IllegalArgumentException("Invalid parameter passed to HC256 init - " + params.getClass().getName());
      }
   }

   public void processBytes(byte[] in, int inOff, int len, byte[] out, int outOff) {
      if (!this.initialised) {
         throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
      } else if (inOff + len > in.length) {
         throw new DataLengthException("input buffer too short");
      } else if (outOff + len > out.length) {
         throw new DataLengthException("output buffer too short");
      } else {
         for(int var6 = 0; var6 < len; ++var6) {
            int var7 = outOff + var6;
            out[var7] = (byte)(in[inOff + var6] ^ this.getByte());
         }

      }
   }

   public void reset() {
      this.idx = 0;
      this.init();
   }

   public byte returnByte(byte in) {
      return (byte)(in ^ this.getByte());
   }
}
